#ifndef QUERY_FORMAT_H
#define QUERY_FORMAT_H
#include <sqlextract/filter/arbval.h>

struct output_formatter_t;

/**
 * Defines an output formatter module. Shouldn't be called directly, but only
 * through output formatter library.
 *
 * Populate a global variable containing this structure with pointers to the
 * methods defining the output formatter.
 */
struct output_formatter_desc_t {
	/**
	 * Create a storage for formatter
	 *
	 * Argument 1: pointer to argument passed to output_formatter_create
	 * Return an internal storage, passed as argument to the other methods.
	 */
	void *(*create)(void *);

	/**
	 * Destroys internal storage
	 *
	 * Argument 1: pointer to internal storage
	 */
	void (*destroy)(void *);

	/**
	 * Begin output of a data set
	 *
	 * Argument 1: pointer to internal storage
	 */
	void (*begin)(void *);

	/**
	 * Called after entire dataset is written.
	 *
	 * Argument 1: pointer to internal storage
	 */
	void (*end)(void *);

	/**
	 * Called before each row should be written.
	 *
	 * Argument 1: pointer to internal storage
	 */
	void (*row)(void *);

	/**
	 * Called for each cell to be written.
	 *
	 * Argument 1: pointer to internal storage
	 * Argument 2: value of the cell
	 */
	void (*field)(void *, struct arbval_t *field);
};

struct output_formatter_t *output_formatter_create(
		struct output_formatter_desc_t *outdesc, void *arg);
void output_formatter_destroy(struct output_formatter_t *outfmt);

void output_formatter_begin(struct output_formatter_t *outfmt);
void output_formatter_end(struct output_formatter_t *outfmt);

void output_formatter_row(struct output_formatter_t *outfmt);
void output_formatter_field(struct output_formatter_t *outfmt,
		struct arbval_t *value);

#endif
