#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <sqlextract/util.h>
#include "options.h"
#include <src/format/json.h>
#include <src/format/default.h>

void usage(const char *progname) {
	printf("\n");
	printf("Usage: %s <filter>\n", progname);
}

static void init_options(void) {
	/*
	 * Initializes the global options struct.
	 * This is also the place to set sane default
	 * values for options, if desired.
	 */
	memset(&options, 0, sizeof(options));

	options.output_formatter = output_formatter_create(&default_formatter, NULL);
}

void deinit_options() {
	free(options.filter);
	output_formatter_destroy(options.output_formatter);
	memset(&options, 0, sizeof(options));
}

#define usage_and_fail() \
		do { usage(progname); exit(EXIT_FAILURE); } while (0);

void parse_options(int argc, char *argv[]) {
	int option_index = 0;
	int help = 0;
	int c = 0;
	char *progname = strdup(argv[0]);
	init_options();
	static struct option longopts[] = {
		{"verbose", no_argument, NULL, 'v'},
		{"help", no_argument, NULL, 'h'}
	};

	if ( argc < 2)
		usage_and_fail()

	while((c = getopt_long(argc, argv, ":vjh", longopts, &option_index)) != -1) {
		switch(c) {
		case 'v':
			options.verbose = 1;
			break;
		case 'h':
			help = 1;
			break;
		case 'j':
			output_formatter_destroy(options.output_formatter);
			options.output_formatter = output_formatter_create(&json_formatter, NULL);
			break;

		case 0: /* long option, strcmp for profit */
			if (longopts[option_index].flag != 0)
				usage_and_fail()
			break;
		default:
			help = 1;
			break;
		}
	}

	if (help) {
		usage(progname);
		exit(EXIT_SUCCESS);
	}

	if (!(optind < argc))
		usage_and_fail()

	options.filter = lf_strdup(argv[optind]);
	free(progname);
}
